/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: csys.c
 control callback functions (for mouse and keyboard)

 code for camera rotation in RoatateView and cSys_MMotion based on
 code and tutorial by: DigiBen <digiben@gametutorials.com>
 found here: http://www.gametutorials.com/Tutorials/OpenGL/OpenGL_Pg1.htm

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>

#include <GL/gl.h>
#include "SDL.h"

#include "genx.h"
#include "csys.h"
#include "gsys.h"
#include "vector.h"
#include "help.h"
#include "minimap.h"
#include "sky.h"

/* both are points in 3D - NOT vectors
   camera view vector is cameraLookAt - cameraPosition */
GLfloat cameraPosition[3] = {0.0f, 20.0f, 0.0f};  
GLfloat cameraLookAt[3] = {0.0f, 20.0f, 1.0f};

int windowIsActive = 1;


/* moving in the direction of camera vector / forward-backward */
static void moveMeFlatZ(float speed) 
{
  float temp[3];
  makeVector(temp,cameraPosition,cameraLookAt);
  reduceToUnit(temp);


  cameraPosition[0] += speed * temp[0];
  cameraPosition[1] += speed * temp[1];
  cameraPosition[2] += speed * temp[2];

  cameraLookAt[0] += speed * temp[0];
  cameraLookAt[1] += speed * temp[1];
  cameraLookAt[2] += speed * temp[2];
}

/* left and right strafe */
static void moveMeFlatX(float speed)
{
   GLfloat temp[] = {0.0f, 0.0f, 0.0f};
   GLfloat temp2[] = {0.0f, 1.0f, 0.0f};
   GLfloat temp3[3];

   makeVector(temp3,cameraPosition,cameraLookAt);
   crossProduct(temp,temp2,temp3);
   reduceToUnit(temp);

   cameraPosition[0] += speed * temp[0];
   cameraPosition[2] += speed * temp[2];
   cameraLookAt[0]   += speed * temp[0];
   cameraLookAt[2]   += speed * temp[2];
}


static void moveMeFlatY(float speed)
{
 cameraPosition[1] += speed;
 cameraLookAt[1] += speed;
}

/* camera rotation - for free mouse look */
static void RotateView(float angle, float x, float y, float z)
{
	float vNewView[3], vView[3];
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

        makeVector(vView,cameraPosition,cameraLookAt);

	vNewView[0] = (cosTheta + (1 - cosTheta) * pow(x,2))	* vView[0];
	vNewView[0]+= ((1 - cosTheta) * x * y - z * sinTheta)	* vView[1];
	vNewView[0]+= ((1 - cosTheta) * x * z + y * sinTheta)	* vView[2];

	vNewView[1]  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView[0];
	vNewView[1] += (cosTheta + (1 - cosTheta) * pow(y,2))	* vView[1];
	vNewView[1] += ((1 - cosTheta) * y * z - x * sinTheta)	* vView[2];

	vNewView[2]  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView[0];
	vNewView[2] += ((1 - cosTheta) * y * z + x * sinTheta)	* vView[1];
	vNewView[2] += (cosTheta + (1 - cosTheta) * pow(z,2))	* vView[2];

	cameraLookAt[0] = cameraPosition[0] + vNewView[0];
	cameraLookAt[1] = cameraPosition[1] + vNewView[1];
	cameraLookAt[2] = cameraPosition[2] + vNewView[2];

}

/*
void cSys_ProcessKey(SDLKey key) 
{
  static char siatka=1;
  static char fog = 1; 
  static char light = 1;
  static char texture = 1;
  static Uint16 count=0;
  char filename[255];

  switch(key) {

  case SDLK_ESCAPE:
	  	SDL_Quit();
  		exit(0);
  		break;

  case SDLK_p:
#ifdef DEBUG
		fprintf(debug,"debug: screenshot disabled now!\n");
#endif
		sprintf(filename,"screen%d.bmp",++count);
      		takeScreenShot(filename);
  		break;

  case SDLK_z:
      RotateView(0.0001f,cameraPosition[0],cameraPosition[1],cameraPosition[2]);
  		break;

  case SDLK_c:
      RotateView(-0.0001f,cameraPosition[0],cameraPosition[1],cameraPosition[2]);
  		break;

  case SDLK_w:
          moveMeFlatZ(30.0f);
  		break;

  case SDLK_s:
          moveMeFlatZ(-30.0f);
  		break;

  case SDLK_a:
          moveMeFlatX(30.0f);
  		break;

  case SDLK_d:
          moveMeFlatX(-30.0f);
  		break;

  case SDLK_m:
         fog *= -1;
         if(fog>0) glEnable(GL_FOG);
         else glDisable(GL_FOG);
  		break;

  case SDLK_t:
         texture *= -1;
         if(texture>0) glEnable(GL_TEXTURE_2D);
         else glDisable(GL_TEXTURE_2D);
  		break;

  case SDLK_l:
         light *= -1;
         if(light>0) glEnable(GL_LIGHTING);
         else glDisable(GL_LIGHTING);
  		break;

  case SDLK_f:
	  	VideoFlags ^= SDL_FULLSCREEN;
	        gSys_Reshape(640,480);
  		break;
	  
  case SDLK_q:
	  siatka*= (-1);
	  if (siatka>0) glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
	  else          glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);	
  		break;

  case SDLK_h:
  	help_mode = !help_mode;
	break;

  case SDLK_u:
  	moveMeFlatY(5.0f);
	break;

  case SDLK_j:
  	moveMeFlatY(-5.0f);
	break;

  case SDLK_i:
  	minimap_mode = !minimap_mode;	
	break;

  default:  
      break;
}

}
*/

/* keyborad callback function */
void cSys_Keyboard()
{
  static char siatka=1;
  static char fog = 1; 
  static char light = 1;
  static char texture = 1;
  static Uint16 count=0;
  char filename[255];
  Uint8 *keystate;
  
  keystate=SDL_GetKeyState(NULL);

  if (keystate[SDLK_ESCAPE]) {
  SDL_Quit();
  exit(0);
  }

  if (keystate[SDLK_p]) {
#ifdef DEBUG
		fprintf(debug,"debug: screenshot disabled now!\n");
#endif
		sprintf(filename,"screen%d.bmp",++count);
      		takeScreenShot(filename);
}

  if (keystate[SDLK_z]) {
      RotateView(0.0001f,cameraPosition[0],cameraPosition[1],cameraPosition[2]);
      }

  if (keystate[SDLK_c]) {
      RotateView(-0.0001f,cameraPosition[0],cameraPosition[1],cameraPosition[2]);
      		}

  if (keystate[SDLK_w]) {
          moveMeFlatZ(10.0f);
          }

  if (keystate[SDLK_s]) {
          moveMeFlatZ(-10.0f);
          }

  if (keystate[SDLK_a]) {
          moveMeFlatX(10.0f);
          }

  if (keystate[SDLK_d]) {
          moveMeFlatX(-10.0f);
          }

  if (keystate[SDLK_m]) {
         fog *= -1;
         if(fog>0) glEnable(GL_FOG);
         else glDisable(GL_FOG);
         }

  if (keystate[SDLK_t]) {
         texture *= -1;
         if(texture>0) glEnable(GL_TEXTURE_2D);
         else glDisable(GL_TEXTURE_2D);
         }

  if (keystate[SDLK_l]) {
         light *= -1;
         if(light>0) glEnable(GL_LIGHTING);
         else glDisable(GL_LIGHTING);
         }

  if (keystate[SDLK_f]) {
	  	VideoFlags ^= SDL_FULLSCREEN;
	        gSys_Reshape(640,480);
	  
    }
  if (keystate[SDLK_q]) {
	  siatka*= (-1);
	  if (siatka>0) glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
	  else          glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);	
      }

  if (keystate[SDLK_h]) {
  	help_mode = !help_mode;
	}

  if (keystate[SDLK_u]) {
  	moveMeFlatY(5.0f);
	}

  if (keystate[SDLK_j]) {
  	moveMeFlatY(-5.0f);
	}

  if (keystate[SDLK_i]) {
  	minimap_mode = !minimap_mode;	
	}
  if (keystate[SDLK_n]) {
  	sky_mode = !sky_mode;	
	}


      
}

 
/* mouse motion callback */
void cSys_MMotion()
{
 int middleX;
 int middleY;
 int x,y;

 float angleY = 0.0f;
 float angleZ = 0.0f;

 static float currentRotX = 0.0f;
 static float lastRotX;

 float up[] = {0.0f, 1.0f, 0.0f};
 float temp[3];
 float vAxis[3];

 SDL_GetMouseState(&x, &y);                        

 middleX = genxScreen->w /2;
 middleY = genxScreen->h /2;

 if ((x==middleX)||(y==middleY))return;

 SDL_WarpMouse(middleX, middleY);                        

 makeVector(temp,cameraPosition,cameraLookAt);


 angleY = (float)( (middleX - x) ) / 1000.0f;		
 angleZ = (float)( (middleY - y) ) / 1000.0f;
 
 lastRotX = currentRotX;
 currentRotX += angleZ;
	if(currentRotX > 1.0f)     
	{
		currentRotX = 1.0f;
		
		if(lastRotX != 1.0f) 
		{
			crossProduct(vAxis,temp,up);
			reduceToUnit(vAxis);
			RotateView( 1.0f - lastRotX, vAxis[0], vAxis[1], vAxis[2]);
		}
	}
	else if(currentRotX < -1.0f)
	{
		currentRotX = -1.0f;
		
		if(lastRotX != -1.0f)
		{
			crossProduct(vAxis,temp,up);
			reduceToUnit(vAxis);
			RotateView( -1.0f - lastRotX, vAxis[0], vAxis[1], vAxis[2]);
		}
	}
	else 
	{	
			crossProduct(vAxis,temp,up);
			reduceToUnit(vAxis);
			RotateView( angleZ, vAxis[0], vAxis[1], vAxis[2]);
		}

	RotateView(angleY, 0, 1, 0);
	
}
