/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: gsys.c

 graphics subsystem - graphics callbacks

 frame per second computing code based on tutorial and code 
 written by: Antnio Ramires Fernandes <antonio.ramires@lighthouse3d.com>
 found here: http://www.lighthouse3d.com/opengl/glut/index.php3?fps

 screen shot function by Ray Kelm  sdl@lokigames.com 

*/

#include <GL/gl.h>
#include <GL/glu.h>

#include "SDL.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "genx.h"
#include "gsys.h"
#include "font.h"
#include "csys.h"
#include "mesh.h"
#include "sky.h"
#include "help.h"
#include "minimap.h"

GLfloat aspect;

GLint takeScreenShot(char *filename)
{
	SDL_Surface *temp;
	unsigned char *pixels;
	int i;
	
	temp = SDL_CreateRGBSurface(SDL_SWSURFACE, genxScreen->w, genxScreen->h, 24,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
	0x000000FF, 0x0000FF00, 0x00FF0000, 0
#else
	0x00FF0000, 0x0000FF00, 0x000000FF, 0
#endif
	);

	if (temp == NULL)
		return -1;

	pixels = malloc(3 * genxScreen->w * genxScreen->h);
	if (pixels == NULL)
	{
		SDL_FreeSurface(temp);
		return -1;
	}

	glReadPixels(0, 0, genxScreen->w, genxScreen->h, GL_RGB, GL_UNSIGNED_BYTE, pixels);

	for (i=0; i<genxScreen->h; i++)
		memcpy(((char *) temp->pixels) + temp->pitch * i, pixels + 3*(genxScreen->w) * ((genxScreen->h)-i-1), (genxScreen->w)*3);
	free(pixels);

	SDL_SaveBMP(temp, filename);
	SDL_FreeSurface(temp);
	return 0;
}


void Enter2D(void) {
  glPushAttrib(GL_ENABLE_BIT);
  glPushAttrib(GL_POLYGON_BIT);
  glDisable(GL_LIGHTING);
  glDisable(GL_CULL_FACE);
  glDisable(GL_FOG);
  glEnable(GL_TEXTURE_2D);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0, genxScreen->w, 0, genxScreen->h, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void Exit2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glPopAttrib();
	glPopAttrib();

}

void gSys_Draw()
{
  static float fps;               /* frames per second */
  static unsigned int timebase=0;
  static unsigned int frame=0;
  int time;
  char strFps[100];

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(cameraPosition[0], cameraPosition[1], cameraPosition[2], 
            cameraLookAt[0], cameraLookAt[1], cameraLookAt[2],
	    0.0f,1.0f,0.0f);


  glPushAttrib(GL_ENABLE_BIT);
  glPushAttrib(GL_POLYGON_BIT);
  glDisable(GL_LIGHTING);
  glDisable(GL_CULL_FACE);
  glEnable(GL_TEXTURE_2D);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f, aspect, 4, TERRAIN_SIZE_XZ*1.43f); 

  if (sky_mode) glCallList(SKY_DISPLAY_LIST);

  glLoadIdentity();
  gluPerspective(45.0f, aspect, 4, FAR_PLANE); 

  glPopAttrib();
  glPopAttrib();

  glCallList(TERRAIN_DISPLAY_LIST);

  Enter2D();

  sprintf(strFps,"fps=%2.2f",fps);
  gl_puts(10,0,genxFont,strFps);
  sprintf(strFps,"genX %s :: www.hope.art.pl/genx/",GENX_VERSION);
  gl_puts(genxScreen->w-260,0,genxFont,strFps);

  if (help_mode==SDL_TRUE) { 
  glCallList(HELP_DISPLAY_LIST);
  }

//  if (minimap_mode==SDL_TRUE) {
//  putMinimap();
//  } 


  Exit2D();

  glFlush();
  SDL_GL_SwapBuffers();

  
  frame++;
  time=SDL_GetTicks();
  if (time-timebase > 1000) {
		fps = frame*1000.0/(time-timebase);
	 	timebase = time;		
		frame = 0;



  }

}

void reInitSDL(int w, int h)
{
genxScreen = SDL_SetVideoMode(w, h,vidInfo->vfmt->BitsPerPixel,VideoFlags);
}



void gSys_Reshape(int w, int h)     
{  

   if (h==0) h=1;
   if (w==0) w=1;

   reInitSDL(w,h);

   aspect=(GLfloat)w/(GLfloat)h;
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();

   gluPerspective(45.0f, aspect, 0.5f, FAR_PLANE); 

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

}
