/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: initgl.c

 initialize opengl and SDL stuff
*/

#include <GL/gl.h>
#include <GL/glu.h>

#include <stdlib.h>

#ifdef DEBUG
#include <stdio.h>
#endif

#include "SDL.h"

#include "genx.h"
#include "initgl.h"

#include "csys.h"
#include "mesh.h"
#include "error.h"
#include "sky.h"
#include "heightmap.h"
#include "font.h"
#include "gsys.h"
#include "help.h"

SDL_Surface *genxScreen;
int VideoFlags = 0;                                     // Video Flags for the Create Window function
SDL_VideoInfo *vidInfo;

/* sets up light and fog */
static void athmosphere()
{
  /* light0 */
  GLfloat light0Position[] = {0.0f,TERRAIN_SIZE_Y*3.0f,0.0f, 1.0f};    

  GLfloat light0Ambient[]  = {0.35f, 0.25f, 0.21f, 1.0f};       
  GLfloat light0Difuse[]   = {0.7f, 0.7f, 0.7f, 0.6f};          
  GLfloat light0Specular[] = {1.0f,1.0f,1.0f,0.9f};  

  /* fog */
  GLfloat fogColor[]  = {1.0f, 1.0f, 1.0f, 1.0f};  

  /* light0 again */
  light0Position[1] = 255*SCALE_Y*1.5f;

  /* lights setup */
  glEnable(GL_LIGHTING);  
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);  

  /* light0 */
  glLightfv(GL_LIGHT0,GL_POSITION,light0Position);
  glLightfv(GL_LIGHT0,GL_AMBIENT,light0Ambient);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,light0Difuse);
  glLightfv(GL_LIGHT0,GL_SPECULAR,light0Specular);
  glEnable(GL_LIGHT0);

  /* fog setup */
  glEnable(GL_FOG);  
  glFogf(GL_FOG_DENSITY, 0.003f);
  glFogi(GL_FOG_MODE, GL_EXP);
  glFogfv(GL_FOG_COLOR, fogColor);
  glHint(GL_FOG_HINT, GL_NICEST);   /* this could be quite slow */
}

/* initializes SDL/opengl things */
GLint initGL(unsigned int width, unsigned int height)
{
#ifdef DEBUG
  fprintf(debug,"debug: initializing SDL window ...\n");
#endif
  
  SDL_Init(SDL_INIT_VIDEO);

  vidInfo = (SDL_VideoInfo*)SDL_GetVideoInfo();

    VideoFlags    = SDL_OPENGL;                         // it's an openGL window
    VideoFlags   |= SDL_HWPALETTE;                      // exclusive access to hardware colour palette
    VideoFlags   |= SDL_RESIZABLE;                      // the window must be resizeable

    if(vidInfo -> hw_available)                      // is it a hardware surface
        VideoFlags |= SDL_HWSURFACE;
    else
        VideoFlags |= SDL_SWSURFACE;

    if(vidInfo -> blit_hw)                            // is hardware blitting available
        VideoFlags |= SDL_HWACCEL;

  if (fullscreen_mode) {
  fprintf(debug,"info: entering fullscreen!\n");
  VideoFlags |= SDL_FULLSCREEN;
  }

  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );      // tell SDL that the GL drawing is going to be double buffered
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, vidInfo->vfmt->BitsPerPixel);         // size of depth buffer 

       // We aren't going to use the stencil buffer.
      SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0 );
      // The next four lines set the bits allocated per pixel -
      // - for the accumulation buffer to zero.
      SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0 );
      SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0 );
      SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0 );
      SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0 );

  gSys_Reshape(width,height);


#ifdef DEBUG
  fprintf(debug,"debug: initializing opengl stuff ...\n");
  fprintf(debug,"debug: GL_RENDERER   = %s\n", (char *) glGetString(GL_RENDERER));
  fprintf(debug,"debug: GL_VERSION    = %s\n", (char *) glGetString(GL_VERSION));
  fprintf(debug,"debug: GL_VENDOR     = %s\n", (char *) glGetString(GL_VENDOR));
#endif

  /* clear buffers */
  glClearColor( 1.0 , 1.0 , 1.0 , 1.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  /* back face removal setup */
  glFrontFace(GL_CW);     
  glEnable(GL_CULL_FACE);  

  /* z-buffering */
  glEnable(GL_DEPTH_TEST);  

  /* smooth shading model */
  glShadeModel(GL_SMOOTH);

  /* textures setup */
  glEnable(GL_TEXTURE_2D); 

  glHint(GL_POLYGON_SMOOTH_HINT,GL_NICEST);
  glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);

  /* blending */
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
  /* sky */
  if (prepareSkyTextures()==-1) { fprintf(debug,"ERROR: loading sky texture failed!\n");
  return(-1);
  }

  putSkyBox(0.0f,0.0f,0.0f,TERRAIN_SIZE_XZ*1.2f, TERRAIN_SIZE_Y*4.0f, TERRAIN_SIZE_XZ*1.2f);

  /* lights and fog */
  athmosphere();

  /* init font */

  if(init_font()==-1) return(-1);


  helpPrepare();

  /* we don't want any cursor to be seen */
  SDL_ShowCursor(0);

  /* we set the mouse pointer position to the center of the screen */
  SDL_WarpMouse(genxScreen->w/2,genxScreen->h/2);

  /* build terrain mesh (display list) -> mesh.c */
  if (makeMesh()==-1) { fprintf(debug,"ERROR: making mesh failed!\n");
  return(-1);
  }

  /* init camera */
  cameraPosition[1] = getHeight(cameraPosition[0],cameraPosition[1])*SCALE_Y-(TERRAIN_SIZE_Y*0.12f);
  cameraLookAt[1] = getHeight(cameraLookAt[0],cameraLookAt[1])*SCALE_Y-(TERRAIN_SIZE_Y*0.12f);


#ifdef DEBUG
  process_errors();
  fprintf(debug,"debug: done with initGL()!\n");
#endif

 return(0);
}
