/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: mesh.c

 builds terrain mesh and texture it

*/

#include <GL/gl.h>
#include <GL/glu.h>

#include "SDL.h"
#include "SDL_image.h"

#include <stdio.h>  
#include <stdlib.h>

#include "genx.h"
#include "mesh.h"
#include "heightmap.h"
#include "normals.h"
#include "error.h"

/* prepare grass texture */
static GLint texturePrepare()
{
  char filename[255];

  SDL_Surface *ImageName;
  GLenum ImageType;

// terrain texture
  sprintf(filename,"%s/%s",TEXTURES_DIR,DEFAULT_TEXTURE);
#ifdef DEBUG
  fprintf(debug,"debug: preparing texture: %s ..\n",filename);
#endif

 ImageName = IMG_Load(filename);
 if (!ImageName) {
 fprintf(debug,"ERROR: loading texture file %s failed with error: %s\n",filename,IMG_GetError());
 return(-1);
 }

#ifdef DEBUG
 else
 fprintf(debug,"debug: file %s ok:)\n",filename);
#endif

  if(ImageName->format->BytesPerPixel == 3) ImageType=GL_RGB;
  else if(ImageName->format->BytesPerPixel == 4) ImageType=GL_RGBA;
  else { fprintf(debug,"ERROR: file: %s is in unsupported pixel format!\n",filename);
         return(-1);
       }

  glGenTextures(1, &terrainTexture);
  glBindTexture(GL_TEXTURE_2D, terrainTexture);
  gluBuild2DMipmaps(GL_TEXTURE_2D, ImageName->format->BytesPerPixel, ImageName->w, ImageName->h, ImageType, GL_UNSIGNED_BYTE, ImageName->pixels);


#ifdef DEBUG
  process_errors();
  SDL_FreeSurface(ImageName);
  fprintf(debug,"debug: done with texturePrepare!\n");
#endif
  return(0);
}

/* puts one vertex of the terrain mesh */
static void putTerrainVertex(unsigned short int x, unsigned short int z)
{
  float temp[3];
  float color[4];
#ifdef DEBUG
  if((x>tSIZE) || (z>tSIZE))
   {
    fprintf(debug,"ERROR: illegal call putTerrainVertex(%d,%d)\n",x,z);
    exit(-1);
   }
#endif

/* calculates texture coordinates */
  glTexCoord2f(x*SCALE_TEXTURE,z*SCALE_TEXTURE);

/* normals are precalculated, so we only need to load one */
  getNormalVector(x,z,temp);
  glNormal3fv(temp);

/* sets color of a vertex - used when texturing is disabled */
  
  getHeightPointColor(x,z,color);
  glColor4fv(color);


/* finally we calculate coordinates of a vertex 
   we want the terrain to be translated so that the center point would be
   in (0,y,0) - and also the y coordinate - look below: */
  glVertex3f(x*SCALE_XZ-(TERRAIN_SIZE_XZ*0.5f),getHeight(x,z)*SCALE_Y-(TERRAIN_SIZE_Y*0.5f),z*SCALE_XZ-(TERRAIN_SIZE_XZ*0.5f));
}

/* main mesh makeing function */
GLint makeMesh()
{
  unsigned short int x,z;

#ifdef DEBUG
  fprintf(debug,"debug: starting makeMesh()!\n");
#endif

  /* preparing texture */
  if (texturePrepare()==-1) return(-1);

#ifdef DEBUG
  fprintf(debug,"debug: compiling mesh display list...\n");
#endif

  /* sets up display list */
  glNewList(TERRAIN_DISPLAY_LIST,GL_COMPILE);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
  glBindTexture(GL_TEXTURE_2D,terrainTexture);

  /* build mesh using triangle strips */
  for(x=0;x<tSIZE-1;x++)
  {
  	glBegin(GL_TRIANGLE_STRIP);
	  for(z=0;z<tSIZE;z++)
  		{
                 putTerrainVertex(x,z);
                 putTerrainVertex(x+1,z);
		 }

	  glEnd();

  }
  glEndList();

#ifdef DEBUG
  process_errors();
  fprintf(debug,"debug: done with makeMesh()!\n");
#endif

 freeNormals();
 return(0);
}
